/*
    Copyright (c) 2002-2009 Tampere University of Technology.

    This file is part of TTA-Based Codesign Environment (TCE).

    Permission is hereby granted, free of charge, to any person obtaining a
    copy of this software and associated documentation files (the "Software"),
    to deal in the Software without restriction, including without limitation
    the rights to use, copy, modify, merge, publish, distribute, sublicense,
    and/or sell copies of the Software, and to permit persons to whom the
    Software is furnished to do so, subject to the following conditions:

    The above copyright notice and this permission notice shall be included in
    all copies or substantial portions of the Software.

    THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
    IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
    FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
    THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
    LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
    FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
    DEALINGS IN THE SOFTWARE.
 */
/**
 * @file TCEAsmPrinter.h
 * 
 * Declaration of TCEAsmPrinter class.
 * 
 * @author Veli-Pekka Jääskeläinen 2007 (vjaaskel-no.spam-cs.tut.fi)
 * @note rating: red
 */

#include <llvm/CodeGen/AsmPrinter.h>

#include <map>

namespace llvm {
class TCEAsmPrinter : public AsmPrinter {
 public:
    TCEAsmPrinter(
        llvm::raw_ostream& o, TargetMachine& tm, const TargetAsmInfo* t);


   virtual ~TCEAsmPrinter();

   virtual const char* getPassName() const {
      return "TCE Assembly Printer";
   }

 private:
   void printOperand(const MachineInstr *MI, int opNum);
   void printMemOperand(
       const MachineInstr *MI, int opNum, const char *Modifier = 0);

   void printCCOperand(const MachineInstr *MI, int opNum);

   bool printInstruction(const MachineInstr *MI);  // autogenerated.
   bool runOnMachineFunction(MachineFunction &F);

   bool doInitialization(Module &M);
   bool doFinalization(Module &M);
   bool PrintAsmOperand(
       const MachineInstr* mi, unsigned opNo,
       unsigned asmVariant, const char* extraCode);

   typedef std::map<const Value*, unsigned> ValueMapTy;
   ValueMapTy NumberForBB_;
   const TargetAsmInfo& asmInfo_;
};
}
